#!/usr/bin/python

# Requires Python 2.3

from optparse import OptionParser, \
        OptionGroup, \
        Option, \
        SUPPRESS_HELP, \
        OptionValueError, \
        TitledHelpFormatter

from ConfigParser import SafeConfigParser as ConfigParser

import textwrap
import sys
import utils

#============================================================================
# Custom Parsers

class DCSHelpFormatter(TitledHelpFormatter):
    """Help formatter for BlueDCS ."""

    # subclassed to handle YES|NO values
    
    def format_option_strings (self, option):
        """Return a comma-separated list of option strings & metavariables."""

        # Overridden to provide formatting for yesno fields.

        if option.type == 'yesno':
            metavar = "YES|NO"
            short_opts = [sopt + metavar for sopt in option._short_opts]
            long_opts = [lopt + "=" + metavar for lopt in option._long_opts]
        elif option.takes_value():
            metavar = option.metavar or option.dest.upper()
            short_opts = [sopt + metavar for sopt in option._short_opts]
            long_opts = [lopt + "=" + metavar for lopt in option._long_opts]
        else:
            short_opts = option._short_opts
            long_opts = option._long_opts

        if self.short_first:
            opts = short_opts + long_opts
        else:
            opts = long_opts + short_opts

        return ", ".join(opts)

class DCSOption(Option):
    """Option parser option class for ArchGenXML parser."""

    # Subclassed to handle YES|NO fields, commalist fields, and to handle
    # sample_config file action.
    
    def check_yesno(option, opt, value):
        yesno={'yes':1, 'y':1, 1:1, '1':1, 'no':None, 'n':None, 0:None, '0':None}

        if yesno.has_key(value.lower()):
            return yesno[value.lower()]
        else:
            choices=", ".join(map(repr, yesno.keys()))
            raise OptionValueError(
                    "option %s: invalid choice: %r (choose from %s)" % 
                    (opt, value, choices))

                    
    #def check_commalist(option, opt, value):
    #    return value
                   
    def take_action(self, action, dest, opt, value, values, parser):
        """Perform option."""

        # Overridden to provide sample_config option and to handle
        # commalist as a list.

        if action == "sample_config":
            print parser.sample_config()
            sys.exit(0)
        elif action == "load_config":
            parser.read_project_configfile(value, values)
        elif action == "append" and self.type == "commalist":
            value = value.split(",")
            values.ensure_value(dest, []).extend(value)
        else:
            Option.take_action(self, action, dest, opt, value, values, parser)
        
    ATTRS=Option.ATTRS + ['section']
    TYPES=Option.TYPES + ('yesno','commalist')
    ACTIONS=Option.ACTIONS + ('sample_config','load_config',)
    TYPE_CHECKER=Option.TYPE_CHECKER
    TYPE_CHECKER['yesno'] = check_yesno
    #TYPE_CHECKER['commalist'] = check_commalist
    TYPED_ACTIONS = Option.TYPED_ACTIONS + ('load_config',)

class DCSOptionParser(OptionParser):
    """Parser for ArchGenXML.

    Handles special data types (yesno, commalist) and generation of sample
    config file.
    """

    def __init__ (self,
                  usage=None,
                  option_list=None,
                  option_class=Option,
                  version=None,
                  conflict_handler="error",
                  description=None,
                  formatter=None,
                  add_help_option=1,
                  prog=None,
                  config_filename=None):
        return OptionParser.__init__(
                self,
                usage,
                option_list,
                DCSOption,
                version,
                conflict_handler,
                description,
                DCSHelpFormatter(),
                add_help_option,
                prog)
        self.config_filename=config_filename
    
    def get_all_options(self):
        """Return all options, recursing into groups."""

        options = [ o for o in self.option_list ]
        for group in self.option_groups:
            options.extend( [ o for o in group.option_list ] )
        return options


    def options_by_section(self):

        sections = {}
        for o in self.get_all_options():
            section = getattr(o, 'section', None)
            if section:
                sections.setdefault(section, [])
                sections[section].append(o)

        return sections
    
    def sample_config(self):
        """Print sample config file.
        """
        for section, options in self.options_by_section().items():
            if  section == 'DEPRECATED':
                continue
            print "[%s]" % section
            print
            for opt in options:
                if opt.help == SUPPRESS_HELP:
                    continue
                name = opt._long_opts[0]
                if not name:
                    continue
                name = name[2:]  # get rid of --
                if name == 'license':
                    # TBD
                    continue
                help_lines = textwrap.wrap(opt.help, 70)
                for line in help_lines:
                    print "## %s" % line
                if opt.type=="yesno":
                    if opt.default:
                        print "#%s: yes" % name
                    else:
                        print "#%s: no" % name
                else:
                    if opt.default != 'NODEFAULT':
                        print "#%s: %s" % (name, opt.default)
                    else:
                        print "#%s: " % name
                # Blank line
                print
            # Blank line
            print
        # Return an empty string, otherwise there's a 'None' on the
        # last printed line.
        return ''    

    def old_sample_config(self):
        """Sample config file"""

        for section, options in self.options_by_section().items():
            print "[%s]" % section
            for opt in options:
                if opt.help == SUPPRESS_HELP: continue
                for name in opt._long_opts:
                    if not name: continue
                    name = name[2:]  # get rid of --
                    help_lines = textwrap.wrap(opt.help, 70)
                    print
                    for line in help_lines:
                        print "#", line
                    if opt.action=="store_true" or opt.action=="store_false":
                        print "#%s" % name
                    elif opt.type=="yesno":
                        print "#%s:yes|no" % name
                    else:
                        print "#%s:%s" % ( name, opt.default or opt.dest.upper() )
            print "\n"


    def read_project_configfile(self, filename, settings):
        """Read project config file into settings."""

        cp = ConfigParser()
        try:
            fname = open(filename,"r")
        except:
            self.print_version()
            print "\nERROR: Can't open project configuration file '%s'!" % filename
            sys.exit(2)

        cp.readfp(fname)
        fname.close()

        # Collect all options
        
        options = self.get_all_options()

        # Walk through options, checking in config file

        for option in options:
            section = getattr(option, 'section', None)
            if section:
                for name in option._long_opts:
                    if not name:
                        continue
                    name = name[2:]
                    if cp.has_option(section, name):
                        option.process(option, cp.get(section, name), settings, parser)


#============================================================================
# BlueDCS Parsing


usage = "usage: %prog [ options ] {start|stop}"
description = "Service for conversion of office documents into HTML."

parser = DCSOptionParser(usage=usage, description=description, version="%prog " + utils.readLocalFile('version.txt'))


#----------------------------------------------------------------------------
# Config File options

group = OptionGroup(parser, "Configuration File Options")

group.add_option("-c", 
        "--cfg", 
        dest="config_file",
        help="Use configuration file",
        action="load_config",
        type="string",
        default="/etc/bluedcs.cfg"
        )

parser.add_option_group(group)


#----------------------------------------------------------------------------
# Parsing Options



group = OptionGroup(parser, "OpenOffice")

group.add_option(
        "--openoffice-port", 
        dest="openoffice_port",
        help="Which port does OpenOffice listen", 
        section="OpenOffice",
        default='2002',
        )

group.add_option(
        "--openoffice-user", 
        dest="openoffice_user",
        help="Which user OpenOffice runs as", 
        section="OpenOffice",
        default='',
        )

group.add_option("--openoffice-autostart",
        help="Automatically launch OpenOffice with bluedcs",
        section="OpenOffice",
        dest="openoffice_autostart",
        type="yesno",
        default=True
        )

group.add_option("--openoffice-command",
        help="command line which start openoffice",
        section="OpenOffice",
        dest="openoffice_command",
        default='/opt/openoffice.org2.0/program/soffice.bin'
        )
        
group.add_option("--before-openoffice-command",
        help="command line which starts before openoffice",
        section="OpenOffice",
        dest="before_openoffice_command",
        default=[],
        action='append'
	
        )
        
group.add_option("--openoffice-options",
        help="command line parameters passed to openoffice, %s will be replaced by the openoffice-port",
        section="OpenOffice",
        dest="openoffice_options",
#        type='commalist',
        action='append',
        default=['-invisible -headless "-accept=socket,host=localhost,port=%s;urp"']
        )
        
parser.add_option_group(group)

group = OptionGroup(parser, "BlueDCS")

group.add_option(
        "--bluedcs-port", 
        dest="bluedcs_port",
        help="Which port does the BlueDCS server listen at", 
        section="BlueDCS",
        default='8888',
        )
        
group.add_option(
        "--pidfile", 
        dest="pidfile",
        help="Location for the PID file", 
        section="BlueDCS",
        default='/var/run/bluedcs.pid',
        )

group.add_option(
        "--logfile", 
        dest="logfile",
        help="Location for the log-file", 
        section="BlueDCS",
        default='/var/log/bluedcs.log',
        )

group.add_option(
        "--errorlogfile", 
        dest="errorlogfile",
        help="Location for the error log-file", 
        section="BlueDCS",
        default='/var/log/bluedcs-error.log',
        )
        

parser.add_option_group(group)


if __name__ == "__main__":
    parser.sample_config()
else:
    settings=parser.get_default_values()
    parser.read_project_configfile(settings.config_file, settings)
    (settings, args) = parser.parse_args(values=settings)
    


